from dataclasses import dataclass
from pathlib import Path

from varidock.pipeline.types import PDB, DeepSurfPocketResult
from varidock.pipeline.stage import Stage
from varidock.broker.deepsurf import predict

@dataclass
class DeepSurfPocketConfig:
    model_dir: Path
    output_dir: Path
    
    # DeepSurf params
    model: str = "orig"  # 'orig' or 'lds'
    f: int = 10  # simplification of points mesh
    T: float = 0.9  # ligandability threshold
    batch: int = 32
    voxel_size: float = 1.0
    protonate: bool = False
    expand: bool = False
    discard_points: bool = False
    seed: int | None = None


class DeepSurfPockets(Stage[PDB, DeepSurfPocketResult]):
    name = "deepsurf_pockets"
    input_type = PDB
    output_type = DeepSurfPocketResult

    def __init__(self, config: DeepSurfPocketConfig):
        self.config = config

    def run(self, input: PDB) -> DeepSurfPocketResult:
        # 2. Run predict.py -p {input.path} -mp {model_dir} -o {output_dir}
        # 3. Parse centers.txt
        # 4. Return PocketSet with conformation and list of PocketCenters 

        target_dir = self.config.output_dir / input.path.stem
        centers_file = target_dir / "centers.txt"
    
        predict(
            prot_file=str(input.path),
            model_path=str(self.config.model_dir),
            output=str(self.config.output_dir),
            model=self.config.model,
            f=self.config.f,
            T=self.config.T,
            batch=self.config.batch,
            voxel_size=self.config.voxel_size,
            protonate=self.config.protonate,
            expand=self.config.expand,
            discard_points=self.config.discard_points,
            seed=self.config.seed,
        )

        return DeepSurfPocketResult(
            pocket_dir=target_dir,
            centers_file=centers_file,
            source_pdb=input,
        )




from __future__ import annotations

from dataclasses import dataclass
from pathlib import Path
from typing import Sequence

from varidock.io import build_af3_input_json
from varidock.jobs import PredictionJob
from varidock.plans import RunPlan
from varidock.runners.base import StructurePredictionRunner

@dataclass(frozen=True)
class AF3Config:
    sif_path: Path
    model_dir: Path
    db_dir: Path
    runner_script: Path

    python_entrypoint: str = "python"

    # Inside-container mount points
    container_input_dir: str = "/root/af_input"
    container_output_dir: str = "/root/af_output"
    container_model_dir: str = "/root/models"
    container_db_dir: str = "/root/public_databases"
    container_runner_dir: str = "/root/runner"

    singularity_args: Sequence[str] = ("--nv",)
    script_args: Sequence[str] = ()

    # @classmethod
    # def from_env(cls) -> "AF3Config":
    #     return cls(
    #         sif_path=Path(os.environ["AF3_SIF_PATH"]),
    #         model_dir=Path(os.environ["AF3_MODEL_DIR"]),
    #         db_dir=Path(os.environ["AF3_DB_DIR"]),
    #         runner_script=Path(os.environ["AF3_RUNNER_SCRIPT"]),
    #     )
    @classmethod
    def from_config(cls, **overrides) -> "AF3Config":
        from varidock.config import VaridockConfig
        cfg = VaridockConfig.load()
        af3 = cfg.af3
        return cls(
            sif_path=overrides.get("sif_path", af3.sif_path),
            model_dir=overrides.get("model_dir", af3.model_dir),
            db_dir=overrides.get("db_dir", af3.db_dir),
            runner_script=overrides.get("runner_script", af3.runner_script),
        )

class AF3Runner(StructurePredictionRunner):
    name = "af3"

    def __init__(self, cfg: AF3Config, seed: int = 1):
        self.cfg = cfg

    
    def plan(self, job: PredictionJob) -> RunPlan:
        input_dir = job.output_dir / "af_input"
        output_dir = job.output_dir / "af_output"

        json_host_path = input_dir / f"{job.name}.json"
        json_container_path = f"{self.cfg.container_input_dir}/{job.name}.json"

        script_path = self.cfg.runner_script.resolve()
        if not script_path.exists():
            raise FileNotFoundError(f"Runner script not found: {script_path}")

        script_host_dir = script_path.parent
        script_name = script_path.name
        script_container_path = f"{self.cfg.container_runner_dir}/{script_name}"

        if not job.input_json_path:
            af3_json = build_af3_input_json(job)
        else:
            af3_json = job.input_json_path.read_text()

        files_text = {
            json_host_path: af3_json,
            output_dir / ".keep": "",  # ensure output dir exists
            input_dir / ".keep": "",  # ensure input dir exists
        }

        argv = ["singularity", "exec"]
        argv += list(self.cfg.singularity_args)

        argv += [
            "--bind",
            f"{input_dir}:{self.cfg.container_input_dir}",
            "--bind",
            f"{output_dir}:{self.cfg.container_output_dir}",
            "--bind",
            f"{self.cfg.model_dir}:{self.cfg.container_model_dir}",
            "--bind",
            f"{self.cfg.db_dir}:{self.cfg.container_db_dir}",
            "--bind",
            f"{script_host_dir}:{self.cfg.container_runner_dir}",
            str(self.cfg.sif_path),
            self.cfg.python_entrypoint,
            script_container_path,
            f"--json_path={json_container_path}",
            f"--model_dir={self.cfg.container_model_dir}",
            f"--db_dir={self.cfg.container_db_dir}",
            f"--output_dir={self.cfg.container_output_dir}",
        ]
        argv += list(self.cfg.script_args)

        files_text[input_dir / "singularity_log.sh"] = "\n".join(argv)
        expected_outputs = [output_dir 
        / job.name.lower()
        / f"{job.name.lower()}_model.cif"] # at least one output

        return RunPlan(
            work_dir=job.output_dir,
            files_text=files_text,
            argv=argv,
            expected_outputs=expected_outputs,
            env=None,
        )


from dataclasses import dataclass
from pathlib import Path

from varidock.pipeline.types import PDB, PDBQT
from varidock.pipeline.stage import Stage
from varidock.broker.adfr import prepare_receptor


@dataclass
class ADFRReceptorPrepConfig:
    output_dir: Path
    # prepare_receptor params
    repairs: str = ""
    charges_to_add: str | None = "gasteiger"
    preserve_charge_types: str | None = None
    cleanup: str = "nphs_lps_waters_nonstdres"
    delete_single_nonstd_residues: bool = False
    unique_atom_names: bool = False
    verbose: bool = False


class ADFRReceptorPrep(Stage[PDB, PDBQT]):
    name = "adfr_receptor_prep"
    input_type = PDB
    output_type = PDBQT

    def __init__(self, config: ADFRReceptorPrepConfig):
        self.config = config

    def run(self, input: PDB) -> PDBQT:
        output_path = self.config.output_dir / f"{input.path.stem}.pdbqt"
        output_path.parent.mkdir(parents=True, exist_ok=True)
        
        prepare_receptor(
            receptor_filename=str(input.path),
            outputfilename=str(output_path),
            repairs=self.config.repairs,
            charges_to_add=self.config.charges_to_add,
            preserve_charge_types=self.config.preserve_charge_types,
            cleanup=self.config.cleanup,
            delete_single_nonstd_residues=self.config.delete_single_nonstd_residues,
            unique_atom_names=self.config.unique_atom_names,
            verbose=self.config.verbose,
        )

        return PDBQT(path=output_path)

# pdb = PDB(path=Path("../../sandbox/test_proteins/P04757/p04757_model.pdb"), source_cif=Path("../../sandbox/test_proteins/P04757/af_output/p04757/p04757_model.cif"))
# cfg = ADFRReceptorPrepConfig(output_dir=Path("../../sandbox/test_proteins/P04757/"),verbose=True)
# ADFRReceptorPrep(cfg).run(pdb)


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""Created on Thu Jan 30 16:00:46 2020.

@author: smylonas

Modified on Sat Feb 6 14:30:00 2026 by abdollahegazy
to include TF2 compatibility.
"""

import numpy as np
from sklearn.cluster import MeanShift


class Bsite_extractor():
    """Extract binding sites from protein surface using clustering.

    This class uses MeanShift clustering to identify potential binding sites
    from ligandability scores computed on protein surface points.

    Attributes:
        T: Ligandability score threshold for filtering surface points.
        ms: MeanShift clustering instance.

    """

    def __init__(self,lig_thres,bw=15):
        """Initialize the binding site extractor.

        Args:
            lig_thres: Ligandability score threshold for filtering surface points.
            bw: Bandwidth parameter for MeanShift clustering. Defaults to 15.

        """
        self.T = lig_thres
        self.ms = MeanShift(bandwidth=bw,bin_seeding=True,cluster_all=False,n_jobs=4)
    
    def _cluster_points(self,prot,lig_scores):
        T_new = self.T
        while sum(lig_scores>=T_new) < 10 and T_new>0.3001:    # at least 10 points with prob>P  and P>=0.3
            T_new -= 0.1 

        filtered_points = prot.surf_points[lig_scores>T_new]
        filtered_scores = lig_scores[lig_scores>T_new]
        if len(filtered_points)<5:
            return () 

        clustering = self.ms.fit(filtered_points)
        labels = clustering.labels_
        
        unique_l,freq = np.unique(labels,return_counts=True)
    
        if len(unique_l[freq>=5])!=0:
            unique_l = unique_l[freq>=5]    # keep clusters with 5 points and more
        else:
            return ()
        
        if unique_l[0]==-1:                 # discard the "unclustered" cluster
            unique_l = unique_l[1:]    
        
        clusters = [(filtered_points[labels==l],filtered_scores[labels==l]) for l in unique_l]  # noqa: E741
        
        return clusters
        
    def extract_bsites(self,prot,lig_scores):
        """Extract binding sites from protein surface and store them.

        Clusters high-ligandability surface points and adds identified
        binding sites to the protein object.

        Args:
            prot: Protein object with surface points and bsite storage methods.
            lig_scores: Array of ligandability scores for each surface point.

        Returns:
            None. Modifies prot in place by adding and writing binding sites.

        """
        clusters = self._cluster_points(prot,lig_scores)
        if len(clusters)==0:
            print('No binding site found')
            return
        for cluster in clusters:
            prot.add_bsite(cluster)
        prot.sort_bsites()
        prot.write_bsites()
        
        
